package com.gic.auth.dto;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.io.Serializable;
import java.util.List;

public class MenuDTO implements Serializable {
    /**
     * 新增页面参数校验分组
     */
    public interface SavePageValid {

    }

    /**
     * 新增gic子页面参数校验分组
     */
    public interface SaveGICChildPageValid {
    }

    /**
     * 编辑gic子页面参数校验分组
     */
    public interface UpdateGICChildPageValid {
    }

    /**
     * 编辑应用页面参数校验分组（应用页面是查询应用列表接口后自动新增）
     */
    public interface UpdateAppPageValid {
    }

    /**
     * 新增应用子页面参数校验分组
     */
    public interface SaveAppChildPageValid {
    }

    /**
     * 编辑应用子页面参数校验分组
     */
    public interface EditAppChildPageValid {
    }

    /**
     * 新增操作项参数校验分组
     */
    public interface SaveOperationItemValid {
    }

    /**
     * 编辑操作项参数校验分组
     */
    public interface UpdateOperationItemValid {
    }

    /**
     *
     */
    private Integer menuId;

    /**
     * 菜单名称
     */
    @NotBlank(message = "页面名称不能为空", groups = { SavePageValid.class, SaveGICChildPageValid.class,
            SaveAppChildPageValid.class, EditAppChildPageValid.class, SaveOperationItemValid.class,
            UpdateGICChildPageValid.class, UpdateOperationItemValid.class })
    private String menuName;

    /**
     * 所在项目
     */
    private String project;

    /**
     * 菜单路由
     */
    @NotBlank(message = "路径URL不能为空", groups = { SaveGICChildPageValid.class, SaveAppChildPageValid.class,
            EditAppChildPageValid.class, SaveOperationItemValid.class, UpdateGICChildPageValid.class,
            UpdateOperationItemValid.class })
    private String menuUrl;

    /**
     * 上级菜单id
     */
    @NotNull(message = "创建子页面，父级ID不能为空", groups = { SaveGICChildPageValid.class, SaveAppChildPageValid.class,
            SaveOperationItemValid.class })
    private Integer parentId;

    /**
     * 0当前页1新开页
     */
    @NotNull(message = "打开方式不能为空", groups = { SavePageValid.class, SaveGICChildPageValid.class,
            UpdateGICChildPageValid.class })
    private Integer target;

    /**
     *
     */
    private String iconUrl;

    /**
     *
     */
    private Integer sort;

    /**
     * 0删除1有效
     */
    private Integer status;

    /**
     *
     */
    private Integer level;

    /**
     * 是否显示在菜单栏的菜单,0否1是
     */
    @NotNull(message = "页面类型（是否作为菜单）不能改为空", groups = { SavePageValid.class, SaveGICChildPageValid.class,
            UpdateGICChildPageValid.class })
    private Integer isShow;

    /**
     * 页面code
     */
    @NotBlank(message = "页面code不能为空", groups = { SavePageValid.class, SaveGICChildPageValid.class,
            UpdateAppPageValid.class, SaveAppChildPageValid.class, EditAppChildPageValid.class,
            SaveOperationItemValid.class, UpdateGICChildPageValid.class, UpdateOperationItemValid.class })
    private String menuCode;

    /**
     * 父级code
     */
    private String parentCode;

    /**
     * 多选，例如_1_2_格式，如果是GIC菜单 1gic标准版  2gic集团版；如果是应用菜单 1：基础班 2:高级版
     */
    @NotBlank(message = "版本不能为空", groups = { SavePageValid.class, SaveGICChildPageValid.class,
            SaveAppChildPageValid.class, SaveOperationItemValid.class, UpdateGICChildPageValid.class,
            EditAppChildPageValid.class, UpdateOperationItemValid.class })
    private String menuVersion;

    /**
     * 0:页面 1:权限项 2:操作项
     */
    private Integer menuType;

    /**
     * 应用版本名称
     */
    @NotBlank(message = "应用版本名称不能为空", groups = { EditAppChildPageValid.class, UpdateOperationItemValid.class })
    private String menuVersionName;

    private List<MenuDTO> children;

    public Integer getMenuId() {
        return menuId;
    }

    public void setMenuId(Integer menuId) {
        this.menuId = menuId;
    }

    public String getMenuName() {
        return menuName;
    }

    public void setMenuName(String menuName) {
        this.menuName = menuName;
    }

    public String getProject() {
        return project;
    }

    public void setProject(String project) {
        this.project = project;
    }

    public String getMenuUrl() {
        return menuUrl;
    }

    public void setMenuUrl(String menuUrl) {
        this.menuUrl = menuUrl;
    }

    public Integer getParentId() {
        return parentId;
    }

    public void setParentId(Integer parentId) {
        this.parentId = parentId;
    }

    public Integer getTarget() {
        return target;
    }

    public void setTarget(Integer target) {
        this.target = target;
    }

    public String getIconUrl() {
        return iconUrl;
    }

    public void setIconUrl(String iconUrl) {
        this.iconUrl = iconUrl;
    }

    public Integer getSort() {
        return sort;
    }

    public void setSort(Integer sort) {
        this.sort = sort;
    }

    public Integer getStatus() {
        return status;
    }

    public void setStatus(Integer status) {
        this.status = status;
    }

    public Integer getLevel() {
        return level;
    }

    public void setLevel(Integer level) {
        this.level = level;
    }

    public Integer getIsShow() {
        return isShow;
    }

    public void setIsShow(Integer isShow) {
        this.isShow = isShow;
    }

    public List<MenuDTO> getChildren() {
        return children;
    }

    public void setChildren(List<MenuDTO> children) {
        this.children = children;
    }

    public String getMenuCode() {
        return menuCode;
    }

    public void setMenuCode(String menuCode) {
        this.menuCode = menuCode;
    }

    public String getParentCode() {
        return parentCode;
    }

    public void setParentCode(String parentCode) {
        this.parentCode = parentCode;
    }

    public String getMenuVersion() {
        return menuVersion;
    }

    public void setMenuVersion(String menuVersion) {
        this.menuVersion = menuVersion;
    }

    public Integer getMenuType() {
        return menuType;
    }

    public void setMenuType(Integer menuType) {
        this.menuType = menuType;
    }

    public String getMenuVersionName() {
        return menuVersionName;
    }

    public void setMenuVersionName(String menuVersionName) {
        this.menuVersionName = menuVersionName;
    }
}
