package com.gic.auth.dao.mapper;

import com.gic.auth.entity.TabSysMenu;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import org.apache.ibatis.annotations.Param;

import java.util.List;

public interface TabSysMenuMapper {
    /**
     * 根据主键删除
     *
     * @param menuId 主键
     * @return 更新条目数
     */
    int deleteByPrimaryKey(Integer menuId);

    /**
     * 插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insert(TabSysMenu record);

    /**
     * 动态插入一条记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int insertSelective(TabSysMenu record);

    /**
     * 根据主键查询
     *
     * @param menuId 主键
     * @return 实体对象
     */
    TabSysMenu selectByPrimaryKey(Integer menuId);

    /**
     * 根据主键动态更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKeySelective(TabSysMenu record);

    /**
     * 根据主键更新记录
     *
     * @param record 实体对象
     * @return 更新条目数
     */
    int updateByPrimaryKey(TabSysMenu record);

    /**
     * 查询菜单树（页面、子页面、权限项）
     * @Title: listMenu 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenu(MenuListQO params);

    /**
     * 获取最大排序值
     * @Title: getMaxSortByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId  父级ID
     * @param project
     * @return  int 
      */
    int getMaxSortByParentId(@Param("parentId") Integer parentId, @Param("project") String project);
    /**
     * 查询页面下的操作项列表数据
     * @Title: listMenuItemByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenuItemByParentId(@Param("parentId") Integer parentId);

    /**
     * 查询页面下的菜单数据
     * @Title: listMenuByParentId 
     * @Description:
      * @author guojuxing
     * @param parentId 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> listMenuByParentId(@Param("parentId") Integer parentId, @Param("project") String project);
    /**
     * 分页查询操作项
     * @Title: pageMenuOperationItem 
     * @Description:
      * @author guojuxing
     * @param params 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> pageMenuOperationItem(MenuOperationItemListQO params);

    /**
     * 查询
     * @Title: selectByIds 
     * @Description:
      * @author guojuxing
     * @param menuIdList 
     * @return  java.util.List<com.gic.auth.entity.TabSysMenu> 
      */
    List<TabSysMenu> selectByIds(@Param("menuIdList") List<Integer> menuIdList);
}