package com.gic.auth.service.impl;

import com.gic.auth.dao.mapper.TabSysUserMapper;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.dto.UserListDTO;
import com.gic.auth.entity.TabSysUser;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.UserService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author guojx
 * @date 2019/7/16 6:52 PM
 */
@Service("userService")
public class UserServiceImpl implements UserService {
    @Autowired
    private TabSysUserMapper tabSysUserMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public int saveUser(UserDTO userDTO) {
        TabSysUser tabUser = EntityUtil.changeEntityNew(TabSysUser.class, userDTO);
        tabSysUserMapper.insert(tabUser);
        return tabUser.getUserId();
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void editUser(UserDTO userDTO) {
        TabSysUser tabUser = EntityUtil.changeEntityNew(TabSysUser.class, userDTO);
        tabSysUserMapper.updateByPrimaryKeySelective(tabUser);
    }

    @Override
    public TabSysUser getUserById(Integer userId) {
        return tabSysUserMapper.selectByPrimaryKey(userId);
    }

    @Override
    public boolean isPhoneRepeat(Integer userId, String phone, Integer enterpriseId) {
        int count = tabSysUserMapper.countByPhone(phone, userId, enterpriseId);
        if (count > 0) {
            return true;
        }
        return false;
    }

    @Override
    public TabSysUser getUserByEnterpriseId(Integer enterpriseId) {
        return tabSysUserMapper.selectByEnterpriseId(enterpriseId);
    }

    @Override
    public List<TabSysUser> listAllUserByPhoneNumber(String phoneNumber) {
        return this.tabSysUserMapper.listAllUserByPhoneNumber(phoneNumber);
    }

    @Override
    public TabSysUser login(String phoneNumber, Integer enterpriseId, String password) {
        return this.tabSysUserMapper.login(phoneNumber, password, enterpriseId);
    }

    @Override
    public Page<UserListDTO> pageUser(UserListQO params) {
        PageHelper.startPage(params.getCurrentPage(), params.getPageSize());
        List<UserListDTO> list = tabSysUserMapper.listUser(params);
        return (Page) list;
    }

    @Override
    public void delete(Integer userId) {
        tabSysUserMapper.deleteById(userId);
    }

    @Override
    public Integer getAllCheckValue(Integer enterpriseId, List<Integer> userIdList) {
        Integer line = tabSysUserMapper.getFirstNotInUserId(enterpriseId, userIdList);
        return line == null ? GlobalInfo.DATA_STATUS_NORMAL : GlobalInfo.DATA_STATUS_DELETE;
    }
}
