package com.gic.auth.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuditedGroupDTO;
import com.gic.auth.entity.TabAuditedGroup;
import com.gic.auth.entity.TabAuditedGroupUserRel;
import com.gic.auth.qo.AuditedGroupQO;
import com.gic.auth.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-09-18 11:26
 */
@Service("auditedGroupApiService")
public class AuditedGroupApiServiceImpl implements AuditedGroupApiService {

    private Logger logger = LogManager.getLogger(AuditedGroupApiServiceImpl.class);

    @Autowired
    private AuditedGroupService auditedGroupService;
    @Autowired
    private AuditedGroupUserRelService auditedGroupUserRelService;
    @Autowired
    private UserService userService;
    @Autowired
    private AuditorAuditedGroupRelService auditorAuditedGroupRelService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    @SuppressWarnings("unchecked")
    public ServiceResponse<Integer> saveOrUpdateAuditedGroup(AuditedGroupDTO auditedGroupDTO) {
        // 判断名是否重复
        boolean repeat = auditedGroupService.validAuditedGroupIsRepeat(auditedGroupDTO.getEnterpriseId(), auditedGroupDTO.getAuditedGroupName(), auditedGroupDTO.getAuditedGroupId());
        if (repeat) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "受审组名称重复");
        }
        // 判断该受审组是否是全选的
        Integer allCheck = userService.getAllCheckValue(auditedGroupDTO.getEnterpriseId(), auditedGroupDTO.getUserIdList());
        auditedGroupDTO.setAllCheck(allCheck);
        Integer id;
        if (auditedGroupDTO.getAuditedGroupId() == null) {
            // save
            id = auditedGroupService.save(auditedGroupDTO);
            auditedGroupDTO.setAuditedGroupId(id);
            if (id == null) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED);
            }
        } else {
            // update
            Integer line = auditedGroupService.update(auditedGroupDTO);
            if (line == 0) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
            }
            id = auditedGroupDTO.getAuditedGroupId();
        }
        this.auditedGroupUserRelService.addOrUpdateRel(auditedGroupDTO);
//        if (line != auditedGroupDTO.getUserIdList().size()) {
//            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
//            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED);
//        }

        return ServiceResponse.success(id);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Integer> delAuditedGroup(Integer auditedGroupId) {
        Integer line = auditedGroupService.delAuditedGroup(auditedGroupId);
        if (line == 0) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        // 删除受审组和管理员的关联关系
        auditedGroupUserRelService.delRel(auditedGroupId);
        // 删除受审组和审核员的关联关系
        auditorAuditedGroupRelService.delRel(auditedGroupId);
        return ServiceResponse.success(line);
    }

    @Override
    public ServiceResponse<Page<AuditedGroupDTO>> listAuditedGroup(AuditedGroupQO auditedGroupQO) {
        com.github.pagehelper.Page<TabAuditedGroup> page = auditedGroupService.listAuditedGroup(auditedGroupQO);
        Page<AuditedGroupDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, AuditedGroupDTO.class);
        if (CollectionUtils.isNotEmpty(resultPage.getResult())) {
            List<Integer> auditedGroupIds = resultPage.getResult().stream().map(AuditedGroupDTO::getAuditedGroupId).collect(Collectors.toList());
            Map<Integer, Integer> map = auditedGroupUserRelService.getUserCountByAuditedGroup(auditedGroupIds);
            for (AuditedGroupDTO dto : resultPage.getResult()) {
                dto.setUserCount(map.get(dto.getAuditedGroupId()));
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<AuditedGroupDTO> getAuditedGroup(Integer auditedGroupId) {
        TabAuditedGroup auditedGroup = auditedGroupService.getAuditedGroup(auditedGroupId);
        if (auditedGroup == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        AuditedGroupDTO auditedGroupDTO = EntityUtil.changeEntityByJSON(AuditedGroupDTO.class, auditedGroup);
        List<TabAuditedGroupUserRel> rels = auditedGroupUserRelService.listAuditedGroup(auditedGroupId);
        if (CollectionUtils.isNotEmpty(rels)) {
            List<Integer> userIds = rels.stream().map(TabAuditedGroupUserRel::getUserId).collect(Collectors.toList());
            auditedGroupDTO.setUserIdList(userIds);
        }
        return ServiceResponse.success(auditedGroupDTO);
    }
}
