package com.gic.auth.service.outer.impl;

import java.util.*;

import com.gic.api.base.commons.Page;
import com.gic.auth.constant.*;
import com.gic.auth.dto.MenuItemDTO;
import com.gic.auth.qo.AuthItemSaveQO;
import com.gic.auth.qo.MenuOperationItemListQO;
import com.gic.auth.service.MenuItemService;
import com.gic.commons.util.PageHelperUtils;
import com.gic.open.api.dto.ApplicationDTO;
import com.gic.open.api.dto.ServePropDTO;
import com.gic.open.api.service.ApplicationApiService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.MenuDTO;
import com.gic.auth.entity.TabSysMenu;
import com.gic.auth.qo.MenuListQO;
import com.gic.auth.service.AuthCodeApiService;
import com.gic.auth.service.MenuApiService;
import com.gic.auth.service.MenuService;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

@Service("menuApiService")
public class MenuApiServiceImpl implements MenuApiService {
    @Autowired
    private MenuService menuService;
    @Autowired
    private AuthCodeApiService authCodeApiService;
    @Autowired
    private ApplicationApiService applicationApiService;
    @Autowired
    private MenuItemService menuItemService;

    @Override
    public ServiceResponse<List<MenuDTO>> getUserMenu(Integer userId, Integer enterpriseId, Integer currentMenu) {
        MenuListQO params = new MenuListQO();
        params.setMenuType(MenuTypeEnum.PAGE.getCode());
        List<TabSysMenu> menuList = this.menuService.listMenu(params);
        int level = 1;
        int parentId = 0;
        if (currentMenu != null) {
            TabSysMenu menu = this.menuService.getMenuById(currentMenu);
            parentId = currentMenu;
            level = menu.getLevel() + 1;
        }
        return ServiceResponse.success(this.treeMenu(menuList, level, parentId));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> getSuperAdminMenu(Integer userId, Integer enterpriseId) {
        return this.getUserMenu(userId, enterpriseId, null);
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuTree(MenuListQO params) {
        if (params.isNeedUpdateAppMenu()) {
            saveAppMenu();
        }
        params.setMenuType(MenuTypeEnum.PAGE.getCode());
        List<TabSysMenu> menuList = this.menuService.listMenu(params);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, menuList));
    }

    @Override
    public ServiceResponse<List<MenuDTO>> listMenuItemByParentId(Integer parentId) {
        TabSysMenu record = menuService.getMenuById(parentId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，页面不存在");
        }
        List<TabSysMenu> menuList = menuService.listMenuItemByParentId(parentId);
        return ServiceResponse.success(EntityUtil.changeEntityListNew(MenuDTO.class, menuList));
    }

    @Override
    public ServiceResponse<Void> saveGICPage(MenuDTO menuDTO) {
        menuDTO.setProject(MenuProjectConstants.DEFAULT_PROJECT);
        return savePage(menuDTO, MenuDTO.SavePageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateGICPage(MenuDTO menuDTO) {
        return updatePage(menuDTO, MenuDTO.SavePageValid.class);
    }

    @Override
    public ServiceResponse<Void> deleteMenu(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        MenuDTO menuDTO = new MenuDTO();
        menuDTO.setMenuId(menuId);
        menuDTO.setStatus(0);
        menuService.updatePage(menuDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveGICChildPage(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveGICChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateGICChildPage(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuDTO.UpdateGICChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateAppPage(MenuDTO menuDTO) {
        return updatePage(menuDTO, MenuDTO.UpdateAppPageValid.class);
    }

    @Override
    public ServiceResponse<Void> saveAppChildPage(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveAppChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> updateAppChildPage(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuDTO.EditAppChildPageValid.class);
    }

    @Override
    public ServiceResponse<Void> setSort(Integer id, Integer sort) {
        TabSysMenu record = menuService.getMenuById(id);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (sort == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "排序值不能为空");
        }
        menuService.setSort(record, sort);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> saveOperationItem(MenuDTO menuDTO) {
        menuDTO.setMenuType(MenuTypeEnum.OPERATION.getCode());
        return saveChildPage(menuDTO, MenuDTO.SaveOperationItemValid.class);
    }

    @Override
    public ServiceResponse<Void> updateOperationItem(MenuDTO menuDTO) {
        return updateChildPage(menuDTO, MenuDTO.UpdateOperationItemValid.class);
    }

    @Override
    public ServiceResponse<Page<MenuDTO>> pageMenuOperationItem(MenuOperationItemListQO params) {
        com.github.pagehelper.Page page = menuService.pageMenuOperationItem(params);
        Page<MenuDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, MenuDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<MenuDTO> getDetailById(Integer menuId) {
        TabSysMenu record = menuService.getMenuById(menuId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(MenuDTO.class, record));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Void> saveAuthItem(AuthItemSaveQO params) {
        TabSysMenu record = menuService.getMenuById(params.getParentId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级页面不存在");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(params, AuthItemSaveQO.SaveAuthItemValid.class);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        //新增权限项
        MenuDTO menuDTO = new MenuDTO();
        menuDTO.setMenuName(params.getMenuName());
        menuDTO.setParentId(params.getParentId());
        setNormalFieldValue(menuDTO, record);
        int menuId = menuService.savePage(menuDTO);

        String operationItemIds = params.getOperationItemIds();
        String[] operationItemArr = operationItemIds.split(SignConstants.COMMA);
        //校验操作项是否存在
        List<Integer> menuIdList = new ArrayList<>(operationItemArr.length);
        for (String str : operationItemArr) {
            menuIdList.add(Integer.parseInt(str));
        }
        List<TabSysMenu> menuList = menuService.selectByIds(menuIdList);
        if (menuList == null || menuList.size() < operationItemArr.length) {
            //回滚
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "操作项数据有误，其中有不存在的操作项");
        }
        //新增权限项、操作项关联表数据
        List<MenuItemDTO> itemDTOList = new ArrayList<>();
        Date now = new Date();
        for (String operationItem : operationItemArr) {
            MenuItemDTO menuItemDTO = new MenuItemDTO();
            menuItemDTO.setItemId(Integer.parseInt(operationItem));
            menuItemDTO.setMenuId(menuId);
            menuItemDTO.setStatus(1);
            menuItemDTO.setCreateTime(now);
            menuItemDTO.setUpdateTime(now);
            itemDTOList.add(menuItemDTO);
        }
        //delete
        menuItemService.deleteByMenuId(menuId);
        //save
        menuItemService.saveFetch(itemDTOList);
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> savePage(MenuDTO menuDTO, Class clazz) {
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        setMenuVersion(menuDTO);
        setNormalFieldValue(menuDTO);
        menuService.savePage(menuDTO);
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> saveChildPage(MenuDTO menuDTO, Class clazz) {
        TabSysMenu record = menuService.getMenuById(menuDTO.getParentId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "父级页面不存在");
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        setNormalFieldValue(menuDTO, record);
        setMenuVersion(menuDTO);

        menuService.savePage(menuDTO);
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> updatePage(MenuDTO menuDTO, Class clazz) {
        return updateNormalPage(menuDTO, clazz, true);
    }

    private ServiceResponse<Void> updateChildPage(MenuDTO menuDTO, Class clazz) {
        return updateNormalPage(menuDTO, clazz, false);
    }

    private ServiceResponse<Void> updateNormalPage(MenuDTO menuDTO, Class clazz, boolean isFirstLevelPage) {
        TabSysMenu record = menuService.getMenuById(menuDTO.getMenuId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，查无数据");
        }
        if (isFirstLevelPage) {
            if (record.getLevel().intValue() != MenuLevelConstants.FIRST_LEVEL) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非第一级页面");
            }
        } else {
            if (record.getLevel().intValue() == MenuLevelConstants.FIRST_LEVEL) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "主键参数有误，非子页面");
            }
        }
        ServiceResponse paramsValid = ValidParamsUtils.allCheckValidate(menuDTO, clazz);
        if (!paramsValid.isSuccess()) {
            return paramsValid;
        }
        menuDTO.setProject(record.getProject());
        setMenuVersion(menuDTO);
        menuService.updatePage(menuDTO);
        return ServiceResponse.success();
    }

    /**
     * 应用第一级页面新增
     */
    private void saveAppMenu() {
        ServiceResponse<List<ApplicationDTO>> appListResponse = applicationApiService.listApplicationAndServeProp();
        Map<String, TabSysMenu> appMenu = getFirstLevelAppMenu();
        if (appListResponse.isSuccess()) {
            List<ApplicationDTO> appList = appListResponse.getResult();
            for (ApplicationDTO app : appList) {
                if (appMenu.containsKey(app.getApplicationId())) {
                    //如果已存在，则更新版本信息
                    MenuDTO menuDTO = new MenuDTO();
                    menuDTO.setMenuName(app.getName());
                    menuDTO.setProject(app.getApplicationId().toString());
                    setMenuVersion(app.getPropDTOList(), menuDTO);
                    menuDTO.setMenuId(appMenu.get(app.getApplicationId()).getMenuId());
                    menuService.updatePage(menuDTO);
                } else {
                    MenuDTO menuDTO = new MenuDTO();
                    menuDTO.setMenuName(app.getName());
                    menuDTO.setProject(app.getApplicationId().toString());
                    setMenuVersion(app.getPropDTOList(), menuDTO);
                    setNormalFieldValue(menuDTO);
                    menuService.savePage(menuDTO);
                }
            }
        }
    }

    /**
     * 第一级页面通用字段值设置
     * @param menuDTO
     */
    private void setNormalFieldValue(MenuDTO menuDTO) {
        //sort
        int maxSort = menuService.getMaxSortByParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID, menuDTO.getProject());
        menuDTO.setSort(maxSort + 1);
        //level
        menuDTO.setLevel(1);

        menuDTO.setParentId(MenuLevelConstants.FIRST_LEVEL_PARENT_ID);
        menuDTO.setMenuType(MenuTypeEnum.PAGE.getCode());
    }

    private void setNormalFieldValue(MenuDTO menuDTO, TabSysMenu record) {
        //project
        menuDTO.setProject(record.getProject());
        //sort
        int maxSort = menuService.getMaxSortByParentId(menuDTO.getParentId(), menuDTO.getProject());
        menuDTO.setSort(maxSort + 1);
        //level
        menuDTO.setLevel(record.getLevel() + 1);
        //父级code
        menuDTO.setParentCode(record.getMenuCode());
    }

    /**
     * 获取应用第一级菜单（应用列表）
     * @return
     */
    private Map<String, TabSysMenu> getFirstLevelAppMenu() {
        MenuListQO params = new MenuListQO();
        params.setIsGIC(2);
        params.setLevel(MenuLevelConstants.FIRST_LEVEL);
        List<TabSysMenu> menuList = menuService.listMenu(params);
        Map<String, TabSysMenu> map = new HashMap<>(16);
        if (menuList != null) {
            for (TabSysMenu menu : menuList) {
                map.put(menu.getProject(), menu);
            }
        }
        return map;
    }

    /**
     * 设置版本ID和版本名称字段值
     * @param serviceList
     * @param menuDTO
     */
    private static void setMenuVersion(List<ServePropDTO> serviceList, MenuDTO menuDTO) {
        if (serviceList == null) {
            return;
        }
        StringBuilder menuVersion = new StringBuilder(SignConstants.UNDERLINE);
        StringBuilder menuVersionName = new StringBuilder();
        for (ServePropDTO propDTO : serviceList) {
            menuVersion.append(propDTO.getPropId()).append(SignConstants.UNDERLINE);
            menuVersionName.append(propDTO.getName()).append(SignConstants.SLASH);
        }
        menuDTO.setMenuVersion(menuVersion.toString());
        String tempMenuVersionName = menuVersionName.toString();
        menuDTO.setMenuVersionName(tempMenuVersionName.substring(0, tempMenuVersionName.length() - 1));
    }

    /**
     * 设置版本ID和版本名称字段值
     * @param menuDTO 必须有project值
     */
    private static void setMenuVersion(MenuDTO menuDTO) {
        if (MenuProjectConstants.DEFAULT_PROJECT.equals(menuDTO.getProject())) {
            menuDTO.setMenuVersion(SignConstants.UNDERLINE
                    + menuDTO.getMenuVersion().replaceAll(SignConstants.COMMA, SignConstants.UNDERLINE)
                    + SignConstants.UNDERLINE);
            String[] menuVersionArr = menuDTO.getMenuVersion().split(SignConstants.UNDERLINE);
            StringBuilder result = new StringBuilder();
            for (String str : menuVersionArr) {
                if (StringUtils.isBlank(str)) {
                    continue;
                }
                result.append(GicVersionEnum.getMessageByCode(Integer.parseInt(str))).append(SignConstants.SLASH);
            }
            String menuVersionStr = result.toString();
            menuDTO.setMenuVersionName(menuVersionStr.substring(0, menuVersionStr.length() - 1));
        } else {
            menuDTO.setMenuVersion(SignConstants.UNDERLINE
                    + menuDTO.getMenuVersion().replaceAll(SignConstants.COMMA, SignConstants.UNDERLINE)
                    + SignConstants.UNDERLINE);
            menuDTO.setMenuVersionName(
                    menuDTO.getMenuVersionName().replaceAll(SignConstants.COMMA, SignConstants.SLASH));
        }
    }

    private List<MenuDTO> treeMenu(List<TabSysMenu> menuList, int level, int parentId) {
        List<MenuDTO> list = null;
        if (level < 5) {
            for (TabSysMenu menu : menuList) {
                if (level == menu.getLevel() && menu.getIsShow().intValue() == 1
                        && menu.getParentId().intValue() == parentId) {
                    MenuDTO menuDTO = EntityUtil.changeEntityByJSON(MenuDTO.class, menu);
                    menuDTO.setChildren(this.treeMenu(menuList, menu.getLevel().intValue() + 1, menuDTO.getMenuId()));
                    if (list == null) {
                        list = new ArrayList<>();
                    }
                    list.add(menuDTO);
                }
            }
        }
        return list;
    }
}
