package com.gic.auth.web.controller;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.Md5Util;
import com.gic.commons.util.ToolUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;

@RestController
public class LoginController {
    private final Logger logger = LogManager.getLogger(LoginController.class);

    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private UserApiService userApiService;

    @RequestMapping("list-enterprise-by-phone")
    public RestResponse listUserEnterprise(String phoneNumber){
        if(StringUtils.isBlank(phoneNumber)){
            return EnterpriseRestResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        ServiceResponse<List<UserDTO>> listServiceResponse = this.userApiService.listUserByPhoneNumber(phoneNumber);
        if(listServiceResponse.isSuccess()){
            List<UserDTO> result = listServiceResponse.getResult();
            if(CollectionUtils.isEmpty(result)){
                return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "用户不存在");
            }else {
                List<Integer> idList = new ArrayList<>();
                for(UserDTO userDTO : result){
                    idList.add(userDTO.getEnterpriseId());
                }
                ServiceResponse<List<EnterpriseDTO>> serviceResponse = this.enterpriseApiService.listEnterpriseByIds(idList);
                if(serviceResponse.isSuccess()){
                    List<EnterpriseDTO> list = serviceResponse.getResult();
                    if(CollectionUtils.isEmpty(list)){
                        return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "用户不存在");
                    }else{
                        return RestResponse.success(list);
                    }
                }
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("login")
    public RestResponse login(String phoneNumber, Integer enterpriseId, String password){
        if(StringUtils.isBlank(password) || StringUtils.isBlank(phoneNumber) || enterpriseId == null){
            return EnterpriseRestResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        Md5Util md5 = new Md5Util();
        // password 自身作为盐值
        if(password.length() != 32) {
            password = md5.encrypt(password + password);
        }
        ServiceResponse<UserDTO> login = this.userApiService.login(phoneNumber, enterpriseId, password);
        if(login.isSuccess()){
            UserDTO userDTO = login.getResult();
            if(userDTO == null){
                return EnterpriseRestResponse.failure(ErrorCode.LOGIN_ERR);
            }
            UserDetail userDetail = new UserDetail();
            userDetail.setUserId(userDTO.getUserId());
            userDetail.setUserInfo(userDTO);
            ServiceResponse<EnterpriseDTO> enterprise = this.enterpriseApiService.getEnterpriseById(enterpriseId);
            if(enterprise.isSuccess()){
                EnterpriseDTO enterpriseDTO = enterprise.getResult();
                if(enterpriseDTO != null){
                    userDetail.setEnterpriseId(enterpriseDTO.getEnterpriseId());
                    userDetail.setEnterpriseInfo(enterpriseDTO);
                    UserDetailUtils.setUserDetail(userDetail);
                    return RestResponse.success(userDetail);
                }
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("logout")
    public RestResponse logout(){
        UserDetailUtils.destory();
        return RestResponse.success();
    }
}
