package com.gic.auth.web.controller;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.qo.UserListQO;
import com.gic.auth.service.UserApiService;
import com.gic.auth.web.utils.UserPasswordUtils;
import com.gic.auth.web.vo.UserDetailVO;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;

@RestController
@RequestMapping("/user")
public class UserController {

    private final Logger logger = LogManager.getLogger(UserController.class);

    @Autowired
    private UserApiService userApiService;

    @RequestMapping("/save-or-update-user")
    public RestResponse editUser(@Validated({ UserDTO.UserQoValid.class }) UserDTO userDTO) {
        Integer operationPassword = userDTO.getOperPasswordType();
        userDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (operationPassword != null && operationPassword.intValue() == 1) {
            //如果是1，则不用验证密码

        } else {
            String password = UserPasswordUtils.validPassword(userDTO.getPasswordType(), userDTO.getPassword());
            userDTO.setPassword(UserPasswordUtils.getEncryptPassword(password));
        }
        ServiceResponse userResult;
        //普通用户
        userDTO.setSuperAdmin(0);
        if (userDTO.getUserId() == null) {
            userResult = userApiService.saveUser(userDTO);
        } else {
            userResult = userApiService.editUser(userDTO);
        }
        if (userResult.isSuccess()) {
            return RestResponse.success();
        } else {
            return EnterpriseRestResponse.failure(userResult);
        }
    }

    /**
     * 用户详情
     * @Title: getUserById 
     * @Description:
      * @author guojuxing
     * @param userId 
     * @return  com.gic.commons.webapi.reponse.RestResponse 
      */
    @RequestMapping("/get-user-by-id")
    public RestResponse getUserById(Integer userId) {
        return ResultControllerUtils.commonResultOne(userApiService.getUserById(userId), UserDetailVO.class);
    }

    @RequestMapping("/delete-user-by-id")
    public RestResponse delete(Integer userId) {
        return ResultControllerUtils.commonResult(userApiService.delete(userId));
    }

    @RequestMapping("/list-user")
    public RestResponse listUser(UserListQO params) {
        params.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(userApiService.pageUser(params));
    }

}
